<?php

namespace Intergo\Helpers;

use Intergo\Helpers\Calculators\SMSCounter;
use libphonenumber\NumberParseException;
use libphonenumber\PhoneNumber;
use libphonenumber\PhoneNumberFormat;
use libphonenumber\PhoneNumberUtil;

class PhoneHelper
{
    const MOBILE_FORMAT = 1;
    const FIXED_LINE_OR_MOBILE = 2;
    const TOLL_FREE = 3;
    const UNKNOWN = 10;

    const NUMBER = 'number';
    const PHONE = 'phone';
    const PHONE_NUMBER_TYPE = 'phoneNumberType';
    const A2_CODE = 'a2_code';
    const COUNTRY_CODE = 'country_code';

    /**
     * @param $phoneNumber
     * @param bool $sanitize
     * @return string
     */
    public static function cleanUpPhoneNumber($phoneNumber, bool $sanitize = false)
    {
        $hasPlusSign = self::startsWithPlusSign($phoneNumber);
        $phoneNumber = self::removeWhitespacesAndNonDigitCharacters($phoneNumber);
        if ($sanitize) {
            $phoneNumber = self::sanitizeToGSM($phoneNumber);
        }

        if (!$hasPlusSign) {
            if ($result = self::checkAndReplaceLeadingZerosWithPlus($phoneNumber)) {
                return $result;
            }
            // If it doesn't start with + sign and neither 00 then prepend + sign
            return '+' . $phoneNumber;
        }

        if ($result = self::checkAndRemoveZerosAfterPlus($phoneNumber)) {
            return $result;
        }
        return $phoneNumber;
    }

    /**
     * Prepend Plus sign on number if plus sign not exists already
     *
     * @param string $phoneNumber
     *
     * @return string
     */
    public static function prependPlusSign(string $phoneNumber)
    {
        return self::startsWithPlusSign($phoneNumber) ? $phoneNumber : '+' . $phoneNumber;
    }

    /**
     * This is to replace Zero on beginning of the phone number with the plus sign
     *
     * @param string $phone
     *
     * @return string|string[]|null
     */
    public static function replaceZeroWithPlusSign(string $phone)
    {
        $phone = preg_replace("/\s+/", "", $phone);
        if (preg_match("~^00\d+$~", $phone)) {
            $phone = preg_replace('/^00?/', "+", $phone);
        } elseif (preg_match("~\+00\d+$~", $phone)) {
            $phone = preg_replace('/^\+00?/', "+", $phone);
        }
        return $phone;
    }

    /**
     * @param $number
     * @return bool
     */
    public static function startsWithPlusSign($number)
    {
        return substr($number, 0, 1) == '+';
    }

    /**
     * @param $number
     * @return string
     */
    public static function removeWhitespacesAndNonDigitCharacters($number)
    {
        return preg_replace("/\s+[^0-9]/", "", $number);
    }

    /**
     * If it starts with 00 then replace with + sign
     *
     * @param $number
     * @return string|false
     */
    public static function checkAndReplaceLeadingZerosWithPlus($number)
    {
        if (preg_match("~^00\d+$~", $number)) {
            return preg_replace('/^00?/', "+", $number);
        }
        return false;
    }

    /**
     * If it starts with +00 (following any number of 0s) remove 0s and leave only + sign
     *
     * @param $number
     * @return string|false
     */
    public static function checkAndRemoveZerosAfterPlus($number)
    {
        if (preg_match("~\+00\d+$~", $number)) {
            return preg_replace('/^\+00?/', "+", $number);
        }
        return false;
    }

    /**
     * @param $number
     * @return array|string|string[]
     */
    private static function sanitizeToGSM($number)
    {
        $smsCounter = new SMSCounter();
        $number = $smsCounter->sanitizeToGSM($number);
        $number = preg_replace('/[\x00-\x08\x0B\x0C\x0E-\x1F\x7F-\x9F]/u', '', $number);
        $number = preg_replace('/[\x00-\x08\x0B\x0C\x0E-\x1F\x7F]/u', '', $number);
        $number = preg_replace('/[\x00-\x1F\x7F-\xA0\xAD]/u', '', $number);
        $number = preg_replace('/[^[:print:]]/', '', $number);
        return str_replace([' ', '-', '(', ')'], '', $number);
    }

    /**
     * @param string $phone
     * @param string|null $defaultPrefix
     * @param bool $requireDetails
     * @return array
     * @throws NumberParseException
     */
    public static function validatePhoneNumber(string $phone, string $defaultPrefix = null, bool $requireDetails = true)
    {
        $phoneWithPlusSign = PhoneHelper::cleanUpPhoneNumber($phone);
        $phoneNumber = self::validatePhoneNumberByGoogle($phoneWithPlusSign, null);
        if (empty($phoneNumber) || (isset($phoneNumber[self::PHONE_NUMBER_TYPE]) && $phoneNumber[self::PHONE_NUMBER_TYPE] !== self::UNKNOWN)) {
            $phoneNumber = self::validatePhoneNumberByGoogle(substr($phoneWithPlusSign, 1), $defaultPrefix);
        }
        return self::getResult($phoneNumber, $requireDetails);
    }

    /**
     * Validate phone number using PhoneNumberUtil
     *
     * @param string $phone
     * @param string|null $defaultPrefix country code
     *
     * @return array
     * @throws NumberParseException
     */
    public static function validatePhoneNumberByGoogle($phone, ?string $defaultPrefix)
    {
        $phoneNumberUtil = PhoneNumberUtil::getInstance();
        /* @var PhoneNumber $number */
        $number = $phoneNumberUtil->parse($phone, $defaultPrefix);
        $phoneNumberType = $phoneNumberUtil->getNumberType($number);
        $countryCode = $phoneNumberUtil->getRegionCodeForNumber($number);

        return [
            self::NUMBER => $number,
            self::PHONE_NUMBER_TYPE => $phoneNumberType,
            self::A2_CODE => $countryCode,
        ];
    }

    /**
     * @param $phoneNumber
     * @param bool $requireDetails
     * @return array
     */
    public static function getResult($phoneNumber, bool $requireDetails = true)
    {
        if (!self::isValidMobileFormat($phoneNumber)) {
            return [];
        }
        /* @var PhoneNumber $number */
        $number = $phoneNumber[self::NUMBER];
        $phone = PhoneNumberUtil::getInstance()->format($number, PhoneNumberFormat::E164);
        $result = [
            self::COUNTRY_CODE => $number->getCountryCode(),
            self::PHONE => self::prependPlusSign($phone),
            self::A2_CODE => $phoneNumber[self::A2_CODE],
        ];
        if ($requireDetails) {
            $result[self::NUMBER] = $number;
        }
        return $result;
    }

    /**
     * @param $phoneNumber
     *
     * @return bool
     */
    public static function isValidMobileFormat($phoneNumber)
    {
        if (!$phoneNumber || !in_array($phoneNumber[self::PHONE_NUMBER_TYPE], self::getValidNumbers(), true)) {
            return false;
        }
        return true;
    }

    /**
     * @return int[]
     */
    private static function getValidNumbers(): array
    {
        return [self::FIXED_LINE_OR_MOBILE, self::MOBILE_FORMAT, self::TOLL_FREE];
    }
}
